--[[
--Title: GonffBar
--Author: Wobin
--Email: Wobster@gmail.com
--Initial Creation Date: 17/05/2006
--Description: A energywatch bar that counts ticks for energy regen and colours itself appropriately if the moused over actionbutton can be executed or not.
--Notes: Heavily borrowed code from Otravi's energy bar, this is basically a direct cut down of his code.
]]--

local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("GonffBar")
GonffBar = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0")

function GonffBar:OnInitialize()
    local _, class = UnitClass("player")

	self.version = self.version..".".. string.sub("$Revision: 31892 $", 12, -3)
	if(class ~= "DRUID" and class ~= "ROGUE") then
		return
	end

	self.surface = AceLibrary("Surface-1.0")
	self.surface:Register("Otravi", "Interface\\AddOns\\GonffBar\\Textures\\otravi")

	self.options = {
		type = "group",
		args = {
			show = {
				type	= "group",
				name	= L["Show"],
				desc	= L["Configurations related to when to show the bar."],
				args	= {
					["while"] = {
						type	= "text",
						name = L["While"],
						desc	= L["Show while..."],
						usage	= '<'..L["Stealthed|Combat|Always|StealthedOnly"]..'>',
						validate = {"Stealthed", "Combat", "Always", "StealthedOnly"},
						set		= function(v)
							self.db.profile.Show = v
							self:StealthCheck()
						end,
						get		= function() return self.db.profile.Show end
					},
					keepticking = {
						type	= 'toggle',
						guiName	= L["Keep Ticking"],
						cmdName = L["keepticking"],
						desc	= L["Toggles whether, in combat to continue ticking or to use realtime energy events. If toggled off, then it will only renew the bar when you actually recieve energy, otherwise it will continue to tick in a two second interval."],
						get		= function() return self.db.profile.KeepTicking end,
						set		= function(v) self.db.profile.KeepTicking = v end
					},
				},
			},
			text = {
				type	= "toggle",
				guiName	= L["Show Text"],
				cmdName = L["showtext"],
				desc	= L["Puts up a textual representation of energy/combopoints."],
				get		= function() return self.db.profile.Text end,
				set		= function(v)
								if(not self.frame and v) then
									self:CreateTextFrame();
								elseif(v) then
									self.frame:Show();
								else
									self.frame:Hide();
								end
								self.db.profile.Text = v; 
							end,
				},
			bar = {
				type	= "group",
				name 	= L["Bar"],
				desc	= L["Configurations related to the bar."],
				args	= {
					width = {
						name 	= L["Width"],
						desc 	= L["Set the width of the bar."],
						type	= "range",
						min		= 25,
						max		= 400,
						order	= 100,
						get		= function() return self.db.profile.Width end,
						set		=
						function(w)
							self.db.profile.Width = w
							self.master:SetWidth(w+9)
							self.master.Bar:SetWidth(w)
							self.master.ComboBar:SetWidth(w)
						end
					},
					height = {
						type	= "range",
						name 	= L["Height"],
						desc 	= L["Set the height of the bar."],
						min		= 5,
						max		= 100,
						order	= 101,
						get		= function() return self.db.profile.Height end,
						set		=
						function(h)
							self.db.profile.Height = h
							self.master:SetHeight(h+10)
							self.master.Spark:SetHeight(h+25)
							self.master.Bar:SetHeight(h)
							self.master.ComboBar:SetHeight(h)
						end
					},
					texture = {
						type = 'text',
						name = L["Texture"],
						desc = L["Set the texture of the bar."],
						get = function() return self.db.profile.Texture end,
						set = function(v)
							self.db.profile.Texture = v
							local tex = self.surface:Fetch(v)
							self:Print("Setting status bar texture to %s", tex)
							self.master.Bar:SetStatusBarTexture(tex)
							self.master.ComboBar:SetStatusBarTexture(tex)
						end,
						validate = self.surface:List()
					},
					border = {
						type	= "toggle",
						name 	= L["Border"],
						desc 	= L["Toggle the border."],
						order	= 102,
						get		= function() return self.db.profile.Border end,
						set		=
						function(v)
							self.db.profile.Border = v
							local ef,es, bg
							if(v) then ef,es = "Interface\\Tooltips\\UI-Tooltip-Border", 16 end
							self.master:SetBackdrop(nil)
							if(self.db.profile.Background) then bg = "Interface\\Tooltips\\UI-Tooltip-Background" else bg = "" end
							self.master:SetBackdrop({
								bgFile = bg, tile = true, tileSize = 16,
								edgeFile = ef or "", edgeSize = es or "",
								insets = {left = 5, right = 5, top = 5, bottom = 5}})
								self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
								self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
							end
						},
						background = {
							type	="toggle",
							name	= L["Background"],
							desc	= L["Toggle the background."],
							order	= 103,
							get		= function() return self.db.profile.Background end,
							set		=
							function(v)
								self.db.profile.Background = v
								local ef,es,bg
								if(self.db.profile.Border) then ef,es = "Interface\\Tooltips\\UI-Tooltip-Border", 16 end
								self.master:SetBackdrop(nil)
								if(v) then bg = "Interface\\Tooltips\\UI-Tooltip-Background" else bg = "" end
								self.master:SetBackdrop({
									bgFile = bg, tile = true, tileSize = 16,
									edgeFile = ef or "", edgeSize = es or "",
									insets = {left = 5, right = 5, top = 5, bottom = 5}})
									self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
									self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
								end
							},
							orientation = {
								type	= "toggle",
								name	= L["Orientation"],
								desc	= L["Toggle from a horizontal view to a vertical one and vice versa."],
								order	= 103,
								map		= { [false] = L["Horizontal"], [true] = L["Vertical"] },
								get		= function() return self.db.profile.Orientation == "Vertical" end,
								set		=
								function(v)
									self.db.profile.Orientation = v and "Vertical" or "Horizontal"
									if(v) then
										self.master.Bar:SetOrientation("VERTICAL")
										self.master.ComboBar:SetOrientation("HORIZONTAL")
										self.master.Spark:Hide()
									else
										self.master.Bar:SetOrientation("HORIZONTAL")
										self.master.ComboBar:SetOrientation("VERTICAL")
										self.master.Spark:Show()
									end
								end
							},
							direction = {
								type	= "toggle",
								name	= L["Direction"],
								desc	= L["Toggle whether you want it to fill or empty the bar."],
								order	= 104,
								guiNameIsMap = true,
								map		= { [false] = L["Fill"], [true] = L["Empty"] },
								get		= function() return self.db.profile.Direction end,
								set		= function(v) self.db.profile.Direction = v end
							},
							level = {
								type = "group",
								name = L["Bar Level"],
								desc = L["Raise or lower the graphical level of the bar so it can be used with other GUI elements."],
								order = 105,
								args = {
									raise = {
										type	= "execute",
										name	= L["Raise"],
										desc	= L["Raise the strata"],
										order	= 100,
										func	=
										function()
											local level = self.master.Bar:GetFrameLevel() + 1
											if(level < 1) then
												level = 1
											end
											self.master:SetFrameLevel(level)
											self.master.Bar:SetFrameLevel(level + 1)
											self.master.ComboBar:SetFrameLevel(level)
											self:Print(L["Level raised"])
											self.db.profile.Level = level
										end
									},
									lower = {
										type	= "execute",
										name	= L["Lower"],
										desc	= L["Lower the strata"],
										order	= 101,
										func	=
										function()
											local level = self.master.Bar:GetFrameLevel() - 1
											if(level < 1) then
												self:Print(L["You cannot lower the level any further."])
												return
											end
											self.master:SetFrameLevel(level)
											self.master.Bar:SetFrameLevel(level + 1)
											self.master.ComboBar:SetFrameLevel(level)
											self:Print(L["Level lowered"])
											self.db.profile.Level = level
										end
									},
								}
							},
							strata = {
								type = "group",
								name = L["Strata"],
								desc = L["Raise the strata level [raise|lower] between BACKGROUND, LOW, HIGH and DIALOG."],
								order = 106,
								args = {
									raise = {
										type	= "execute",
										name	= L["Raise"],
										desc	= L["Raise the Bar"],
										order	= 100,
										func	=
										function()
											if(self.db.profile.Strata < table.getn(self.k.Strata)) then
												self.db.profile.Strata = self.db.profile.Strata + 1
											else
												self:Print(L["You cannot raise the strata any further."])
											end
											self.master:SetFrameStrata(self.k.Strata[self.db.profile.Strata])
											self:Print(L["Strata set to "]..self.k.Strata[self.db.profile.Strata])
										end
									},
									lower = {
										type	= "execute",
										name	= L["Lower"],
										desc	= L["Lower the Bar"],
										order	= 101,
										func	=
										function()
											if(self.db.profile.Strata > 1) then
												self.db.profile.Strata = self.db.profile.Strata - 1
											else
												self:Print(L["You cannot lower the strata any further."])
											end
											self.master:SetFrameStrata(self.k.Strata[self.db.profile.Strata])
											self:Print(L["Strata set to "]..self.k.Strata[self.db.profile.Strata])
										end
									},
								}
							},
						},
					},
					colours = {
						type = "group",
						name = L["Colours"],
						desc = L["Set the bar color."],
						args = {
							energy = {
								type	= "color",
								name	= L["Energy"],
								desc 	= L["Sets the color of the bar for standard energy ticks."],
								get		= function() return self.db.profile.Colours.Energy.r, self.db.profile.Colours.Energy.g, self.db.profile.Colours.Energy.b, self.db.profile.Colours.Energy.a end,
								set		= function(r,g,b,a)
									self.db.profile.Colours.Energy.r = r
									self.db.profile.Colours.Energy.g = g
									self.db.profile.Colours.Energy.b = b
									self.db.profile.Colours.Energy.a = a
								end,
								hasAlpha = true
							},
							enoughEnergy = {
								type	= "color",
								name	= L["Enough Energy"],
								desc 	= L["Sets the color of the bar for when you have enough energy for that action."],
								get		= function() return self.db.profile.Colours.EnoughEnergy.r, self.db.profile.Colours.EnoughEnergy.g, self.db.profile.Colours.EnoughEnergy.b, self.db.profile.Colours.EnoughEnergy.a end,
								set		= function(r,g,b,a)
									self.db.profile.Colours.EnoughEnergy.r = r
									self.db.profile.Colours.EnoughEnergy.g = g
									self.db.profile.Colours.EnoughEnergy.b = b
									self.db.profile.Colours.EnoughEnergy.a = a
								end,
								hasAlpha = true
							},
							comboPoints = {
								type	= "color",
								name	= L["Combo Points"],
								desc 	= L["Sets the color of the bar for your combopoint accumulation."],
								get		= function() return self.db.profile.Colours.ComboPoints.r, self.db.profile.Colours.ComboPoints.g, self.db.profile.Colours.ComboPoints.b, self.db.profile.Colours.ComboPoints.a end,
								set		= function(r,g,b,a)
									self.db.profile.Colours.ComboPoints.r = r
									self.db.profile.Colours.ComboPoints.g = g
									self.db.profile.Colours.ComboPoints.b = b
									self.db.profile.Colours.ComboPoints.a = a
								end,
								hasAlpha = true
							},
							comboPointsFull = {
								type	= "color",
								name	= L["Full Combo Points"],
								desc 	= L["Sets the color of the bar when you hit five combopoints."],
								get		= function() return self.db.profile.Colours.FullComboPoints.r, self.db.profile.Colours.FullComboPoints.g, self.db.profile.Colours.FullComboPoints.b, self.db.profile.Colours.FullComboPoints.a end,
								set		= function(r,g,b,a)
									self.db.profile.Colours.FullComboPoints.r = r
									self.db.profile.Colours.FullComboPoints.g = g
									self.db.profile.Colours.FullComboPoints.b = b
									self.db.profile.Colours.FullComboPoints.a = a
								end,
								hasAlpha = true
							},
						}
					}
				}
			}
	self:RegisterChatCommand(L["AceConsole-commands"], self.options)
	self:RegisterDB("GonffBarDB")
	self:RegisterDefaults("profile", {
			Show		= "Always",
			Colours		= {
				Energy		= {r = 0.5, g = 1, b = 0.7, a=0.5},
				EnoughEnergy	= {r = 0.9, g = 0.9, b = 0.4, a=1},
				ComboPoints = {r = 1, g = 0.4, b = 0, a=1},
				FullComboPoints= {r = 1, g = 0, b = 0.4, a=1},
			},
			Width		= 225,
			Height		= 25,
			Border		= true,
			Background	= true,
			Orientation = "Horizontal",
			Direction	= "Empty",
			KeepTicking = false,
			Text		= false,
			Level		= 3,
			Strata		= 1,
			Texture         = "Otravi",
			})

	self.k = {}

	self.k.StealthAura = { ["Stealth"] = true, ["Prowl"] = true }
	self.k.Class = class
	self.k.Strata = {"BACKGROUND", "LOW", "MEDIUM", "HIGH", "DIALOG"}
	self.k.Orientation = {Horizontal = "Vertical", Vertical = "Horizontal"}
	self.k.Showing =  {Stealthed = 2, Combat = 1, Always = 3, StealthedOnly = 2}


	self.var = {
		maxValue=0,
		ActionButtons = {},
		currentEnergy = 0,
	}
	self:CreateCB()
end

function GonffBar:GetFrames()
	local time = GetTime()
	local frame = EnumerateFrames()
	local count = 1
	local found = 0
	while(frame) do
		local name = frame:GetName()
		if(name) then
            local slot
            slot = select(3,string.find(name, "BT3Button(%d+)$"))
            if not slot then
                slot = select(3,string.find(name, "Action.-Button.-(%d+)$"))
            end
			if(slot) then
				found = found + 1
				GonffBar.var.ActionButtons[frame] = slot
			end
		end
		frame = EnumerateFrames(frame)
			count = count + 1
	end
	self.var.found = found
	self.var.loadTime = GetTime() - time
	self.var.loadFrames = count
end

function GonffBar:Disable()
	self:UnregisterEvent("UNIT_ENERGY")
	self:UnregisterEvent("PLAYER_COMBO_POINTS")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_AURAS_CHANGED")
end

function GonffBar:OnEnable()
	local _, class = UnitClass("player")
	if(class == "DRUID" or class == "ROGUE") then

		self.options.args.bar.args.texture.validate = self.surface:List()
        self:ScheduleEvent(self.GetFrames, 3, self)
		self:Events()
		self:RestorePosition()
	end
end

function GonffBar:Events()
	self:RegisterEvent("UNIT_ENERGY", "SpellStart")
	self:RegisterEvent("PLAYER_COMBO_POINTS", "UpdatePoints")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "StealthCheck")
	self:RegisterEvent("PLAYER_AURAS_CHANGED", "StealthCheck")
	self:RegisterEvent("Surface_Registered", function()
		self.options.args.bar.args.texture.validate = self.surface:List()
	end)
end

function GonffBar:CreateTextFrame()
	local f, _ = GameFontHighlightSmall:GetFont()
	
	self.frame = CreateFrame("Frame", "GonffTextFrame", self.master)
	self.frame:SetWidth(86)
	self.frame:SetHeight(32)
	self.frame:SetPoint("CENTER", self.master, "CENTER")
--	self.frame:SetBackdrop({
--		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
--		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
--		insets = {left = 4, right = 4, top = 4, bottom = 4},
--	})
--	self.frame:SetBackdropBorderColor(0, 0, 0)
--	self.frame:SetBackdropColor(0, 0, 0)
	
	self.frame.CP = self.frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	self.frame.CP:SetPoint("CENTER", self.frame, "CENTER", 0, -6)
	self.frame.CP:SetShadowOffset(.8, -.8)
	self.frame.CP:SetShadowColor(0, 0, 0, 1)
	
	self.frame.Energy = self.frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	self.frame.Energy:SetPoint("CENTER", self.frame, "CENTER", 0, 6)
	self.frame.Energy:SetShadowOffset(.8, -.8)
	self.frame.Energy:SetShadowColor(0, 0, 0, 1)
	self.frame:SetFrameLevel(self.db.profile.Level + 2);
end

function GonffBar:RestorePosition()
	self.master:ClearAllPoints()
	if (not self.db.profile.Position or not self.db.profile.Position[1]) then
		self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		self:SavePosition()
	else
		local x,y = self.db.profile.Position[1], self.db.profile.Position[2]
		self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
	end
end

function GonffBar:CreateCB()
	self.master = CreateFrame("Button", "GonffBarFrame", UIParent)
	self.master:Hide()
	self.master:SetScript("OnUpdate", self.OnUpdate)
	self.master:SetMovable(true)
	self.master:EnableMouse(true)
	self.master:RegisterForDrag("LeftButton")
	self.master:RegisterForClicks("RightButtonUp")
	self.master:SetScript("OnDragStart", function() if IsControlKeyDown() then self.master:StartMoving() end end)
	self.master:SetScript("OnDragStop", function() self.master:StopMovingOrSizing() self:SavePosition() end)
	self.master:SetScript("OnClick", function() if IsControlKeyDown() then dewdrop:Open(self.master) end end)
	self.master:SetWidth(self.db.profile.Width + 9)
	self.master:SetHeight(self.db.profile.Height + 10)
	self.master:SetFrameStrata(self.k.Strata[self.db.profile.Strata])
	self.master:SetFrameLevel(self.db.profile.Level)

	local ef, es, bg
	if(self.db.profile.Border) then ef,es = "Interface\\Tooltips\\UI-Tooltip-Border", 16 end

	if(GonffBar.db.profile.Background) then bg = "Interface\\Tooltips\\UI-Tooltip-Background" end
	GonffBar.master:SetBackdrop({
		bgFile = bg or "", tile = true, tileSize = 16,
		edgeFile = ef or "", edgeSize = es or "",
		insets = {left = 5, right = 5, top = 5, bottom = 5}})
	GonffBar.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	GonffBar.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

	local bar = CreateFrame("StatusBar", nil, self.master)
	bar:ClearAllPoints()
	bar:SetPoint("CENTER", self.master, "CENTER", 0, 0)
	bar:SetWidth(self.db.profile.Width)
	bar:SetHeight(self.db.profile.Height)
	bar:SetOrientation(string.upper(self.db.profile.Orientation))
	bar:SetFrameLevel(self.db.profile.Level + 2)
	bar:SetStatusBarTexture(self.surface:Fetch(self.db.profile.Texture))
	self:Print("Setting status bar texture to %s", self.surface:Fetch(self.db.profile.Texture))
	self.master.Bar = bar

	local combobar = CreateFrame("StatusBar", nil, self.master)
	combobar:ClearAllPoints()
	combobar:SetPoint("CENTER", self.master, "CENTER", 0,0 )
	combobar:SetWidth(self.db.profile.Width)
	combobar:SetHeight(self.db.profile.Height)
	combobar:SetStatusBarTexture(self.surface:Fetch(self.db.profile.Texture))
	combobar:SetOrientation(string.upper(self.k.Orientation[self.db.profile.Orientation]))
	combobar:SetStatusBarColor(1, 0.5,0)
	combobar:SetMinMaxValues(0,5)
	combobar:SetValue(0)
	combobar:SetFrameLevel(self.db.profile.Level + 1)
	self.master.ComboBar = combobar

	local spark = bar:CreateTexture(nil, "OVERLAY")
	spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	spark:SetWidth(16)
	spark:SetHeight(self.db.profile.Height + 25)
	spark:SetBlendMode("ADD")
	self.master.Spark = spark
	if(self.db.profile.Text) then
		self:CreateTextFrame()
	end
	self:StartTimer()

	dewdrop:Register(self.master,
								 'children', function()
									 dewdrop:FeedAceOptionsTable(self.options)
								 end,
								 'point', function() return "BOTTOMLEFT", "TOPLEFT" end)
end

function GonffBar:PLAYER_REGEN_ENABLED()
	if(self.k.Showing[self.db.profile.Show] < 3) then
		self.master:Hide()
	end
end

function GonffBar:PLAYER_REGEN_DISABLED()
	self:StealthCheck()
end

function GonffBar:PLAYER_AURAS_CHANGED()
	self:StealthCheck()
end

function GonffBar:UpdatePoints()
	local points = GetComboPoints()
	if(points < 5) then
		if(self.frame) then self.frame.CP:SetTextColor(1, 1, 0) end
		self.master.ComboBar:SetStatusBarColor(self.db.profile.Colours.ComboPoints.r, self.db.profile.Colours.ComboPoints.g, self.db.profile.Colours.ComboPoints.b, self.db.profile.Colours.ComboPoints.a)
	else
		if(self.frame) then self.frame.CP:SetTextColor(1, 0, 0) end
		self.master.ComboBar:SetStatusBarColor(self.db.profile.Colours.FullComboPoints.r, self.db.profile.Colours.FullComboPoints.g, self.db.profile.Colours.FullComboPoints.b, self.db.profile.Colours.FullComboPoints.a)
	end
	if(self.frame) then self.frame.CP:SetText(string.format("%d of %d", points, MAX_COMBO_POINTS)) end
	self.master.ComboBar:SetValue(GetComboPoints())
end

function GonffBar:StealthCheck()
	local _, _, CatFormed = GetShapeshiftFormInfo(3) -- Hard coded catform?
	if(self.k.Class == "ROGUE" or (CatFormed and self.k.Class == "DRUID")) then
		if(self.k.Showing[self.db.profile.Show] < 3)  then
			if(self.k.Showing[self.db.profile.Show] == 2) then
				local i = 1
				while (UnitBuff("player", i)) do
					name, _, _, _= UnitBuff("player", i)
					
					if(self.k.StealthAura[name]) then
						self.master:Show()
						self.var.Stealthed = true
						return
					else
						i = i + 1
					end
				end

				if(self.var.Stealthed) then
					if(not UnitAffectingCombat("player")) then
						self.master:Hide()
					end
					self.var.Stealthed = false
				end
				if(self.db.profile.Show == "StealthedOnly" and UnitAffectingCombat("player")) then
					self.master:Hide()
					return
				end
			end
			if(not UnitAffectingCombat("player")) then
				self.master:Hide()
				return
			end
		end
		self.master:Show()
	else
		self.master:Hide()
	end
end

function GonffBar:SpellStart(target)
	if(target ~= "player") then return end
	if(self.k.Showing[self.db.profile.Show] < 3 and not UnitAffectingCombat("player")) then return end
	if(self.db.profile.Show == "StealthedOnly" and UnitAffectingCombat("player")) then return end
	if(UnitMana("player") <= self.var.currentEnergy) then
		self.var.currentEnergy = UnitMana("player")
		return
	end
	self.var.currentEnergy = UnitMana("player")
	local mc, mx = UnitMana("player"), UnitManaMax("player")
	if(self.frame) then
		if(mc == mx) then
			self.frame.Energy:SetTextColor(0, 1, 0)
		else
			self.frame.Energy:SetTextColor(1, 1, 0)
		end
		self.frame.Energy:SetText(string.format("%d of %d", mc, mx))
	end
	self:StartTimer(1)
end

function GonffBar:StartTimer(regen)
	self.var.startTime = GetTime()
	self.var.maxValue = self.var.startTime + 2
	self.master.Bar:SetMinMaxValues(self.var.startTime, self.var.maxValue )
	self.master.Bar:SetValue(self.db.profile.Direction and self.var.maxValue or 0)
	self.master:SetAlpha(1)

	self.var.fadeOut 	= nil

	self.master:Show()
	if(self.master.Bar:GetOrientation() == "HORIZONTAL") then
		self.master.Spark:Show()
	else
		self.master.Spark:Hide()
	end
	self.var.LastTimer = GetTime()
end


function GonffBar:OnUpdate()
	local self = GonffBar

	local n = self.db.profile.Direction and self.var.maxValue - GetTime() or GetTime() - self.var.startTime
	if  (n < 0 or n > 2 and (UnitMana("player") == UnitManaMax("player") or self.db.profile.KeepTicking)) then
		return self:StartTimer()
	end

	self.master.Bar:SetValue(self.var.startTime + n)

	local w = self.master.Bar:GetWidth()
	sp = ((n) / (self.var.maxValue - self.var.startTime)) * w
	if( sp < 0 ) then sp = 0 end
	if( sp > w ) then sp = w end
	self.master.Spark:SetPoint("CENTER", self["master"]["Bar"], "LEFT", sp, 0)

	local canDo, hasNoEnergy = IsUsableAction(self.var.ActionButtons[GetMouseFocus()] or -1)

	if(hasNoEnergy or not canDo) then
		self.master.Bar:SetStatusBarColor(self.db.profile.Colours.Energy.r,self.db.profile.Colours.Energy.g,self.db.profile.Colours.Energy.b,self.db.profile.Colours.Energy.a)
	else
		self.master.Bar:SetStatusBarColor(self.db.profile.Colours.EnoughEnergy.r, self.db.profile.Colours.EnoughEnergy.g, self.db.profile.Colours.EnoughEnergy.b, self.db.profile.Colours.EnoughEnergy.a)
	end

	if((UnitMana("player") == UnitManaMax("player") and (self.k.Showing[self.db.profile.Show] == 1 and not UnitAffectingCombat("player")) or
		(not self.var.Stealthed and self.k.Showing[self.db.profile.Show] == 2 and not UnitAffectingCombat("player")))) then
		self.var.fadeOut = true
	end
	if(self.var.fadeOut) then
		local a = this:GetAlpha() - .05

		if (a > 0) then
			self.master:SetAlpha(a)
		else
			self.var.fadeOut = nil
			self.master:Hide()
			self.master:SetAlpha(1)
		end
	end
end

function GonffBar:SavePosition()
	local x,y = self.master:GetLeft(), self.master:GetTop()
	self.db.profile.Position = { x,y }
end

function GonffBar:Stats()
	self:Print("Number of frames scanned: "..self.var.loadFrames.." in "..self.var.loadTime.." seconds. Found "..self.var.found)
end

function GonffBar:Frames(start, finish)
    for i=start,finish do
        self:Print(i..":"..self.var.ActionButtons[i]:GetName())
    end
end
